from textual_englyph import EnGlyph
from textual.app import App, ComposeResult

class Test(App):
    DEFAULT_CSS = """
    EnGlyph {
        height: 4;
        color: green;
        text-style: strike;
        }
    """

    def compose(self) -> ComposeResult:
        #yield EnGlyph("Hello [blue]Textual!")
        yield EnGlyph("Hello [u blue]Textual!")
        #yield EnGlyph("[u blue]Textual!")

if __name__ == "__main__":
    Test().run(inline=True, inline_no_clear=True)



from textual_englyph import EnGlyph
from textual.app import App, ComposeResult

class Test(App):

    phrase_list = ["Hello Textual.", "It's the world!"]
    basis_list = [(2,4),(2,3),(2,2),(1,2),(1,1)]
    use_pips = False

    test_widget = EnGlyph("From EnGlyph:")

    def compose(self) -> ComposeResult:
        yield self.test_widget

    def on_mount(self):
        self.set_interval( 2.2, self.next_render )

    def next_render(self):
        this_basis = self.basis_list.pop(0)
        self.basis_list.append( this_basis )

        self.test_widget.update( renderable=self.phrase_list[0], basis=this_basis, pips=self.use_pips )
        self.set_timer(1, self.rerender)

    def rerender(self):
        self.test_widget.update( renderable=self.phrase_list[1], basis=self.basis_list[-1], pips=self.use_pips )
        if self.basis_list[-1] == (1,1):
            self.use_pips = not self.use_pips
            if not self.use_pips: 
                self.test_widget.update( renderable="GoodBye!", basis=self.basis_list[-1], pips=self.use_pips )
                self.set_timer(1, self.app.exit)


if __name__ == "__main__":
    Test().run()




from textual.app import App, ComposeResult
from textual.containers import Grid
from textual.widgets import Label

class Test(App):

    DEFAULT_CSS = """
    Grid {
        grid-size: 2 1;
    }
    Label {
        text-align: right;
    }
    """

    def compose(self) -> ComposeResult:
        yield Grid(
            Label("Hello"),
            Label("World!")
        )

if __name__ == "__main__":
    Test().run()



from textual.strip import Strip
from rich.console import Console
from rich.segment import Segment
from rich.style import Style
#from rich.traceback import install
#install()
#raise ValueError("My message")

from typing import List
from PIL import Image, ImageDraw, ImageFont
from importlib import resources

class ToGlyxels():
    """Glyph pixels to enable user specified font based string rendering via PIL"""

    #full infill glyxel(glyph pixel) look up table, columns x rows
    full_glut = [[],["","",""],["","","","",""]]
    full_glut[1][1] = " █"
    full_glut[1][2] = " ▀▄█"
    full_glut[2][2] = " ▘▝▀▖▌▞▛▗▚▐▜▄▙▟█"
    full_glut[2][3] = " 🬀🬁🬂🬃🬄🬅🬆🬇🬈🬉🬊🬋🬌🬍🬎🬏🬐🬑🬒🬓▌🬔🬕🬖🬗🬘🬙🬚🬛🬜🬝🬞🬟🬠🬡🬢🬣🬤🬥🬦🬧▐🬨🬩🬪🬫🬬🬭🬮🬯🬰🬱🬲🬳🬴🬵🬶🬷🬸🬹🬺🬻█"
    full_glut[2][4] = " 𜺨𜺫🮂𜴀▘𜴁𜴂𜴃𜴄▝𜴅𜴆𜴇𜴈▀𜴉𜴊𜴋𜴌🯦𜴍𜴎𜴏𜴐𜴑𜴒𜴓𜴔𜴕𜴖𜴗𜴘𜴙𜴚𜴛𜴜𜴝𜴞𜴟🯧𜴠𜴡𜴢𜴣𜴤𜴥𜴦𜴧𜴨𜴩𜴪𜴫𜴬𜴭𜴮𜴯𜴰𜴱𜴲𜴳𜴴𜴵🮅𜺣𜴶𜴷𜴸𜴹𜴺𜴻𜴼𜴽𜴾𜴿𜵀𜵁𜵂𜵃𜵄▖𜵅𜵆𜵇𜵈▌𜵉𜵊𜵋𜵌▞𜵍𜵎𜵏𜵐▛𜵑𜵒𜵓𜵔𜵕𜵖𜵗𜵘𜵙𜵚𜵛𜵜𜵝𜵞𜵟𜵠𜵡𜵢𜵣𜵤𜵥𜵦𜵧𜵨𜵩𜵪𜵫𜵬𜵭𜵮𜵯𜵰𜺠𜵱𜵲𜵳𜵴𜵵𜵶𜵷𜵸𜵹𜵺𜵻𜵼𜵽𜵾𜵿𜶀𜶁𜶂𜶃𜶄𜶅𜶆𜶇𜶈𜶉𜶊𜶋𜶌𜶍𜶎𜶏▗𜶐𜶑𜶒𜶓▚𜶔𜶕𜶖𜶗▐𜶘𜶙𜶚𜶛▜𜶜𜶝𜶞𜶟𜶠𜶡𜶢𜶣𜶤𜶥𜶦𜶧𜶨𜶩𜶪𜶫▂𜶬𜶭𜶮𜶯𜶰𜶱𜶲𜶳𜶴𜶵𜶶𜶷𜶸𜶹𜶺𜶻𜶼𜶽𜶾𜶿𜷀𜷁𜷂𜷃𜷄𜷅𜷆𜷇𜷈𜷉𜷊𜷋𜷌𜷍𜷎𜷏𜷐𜷑𜷒𜷓𜷔𜷕𜷖𜷗𜷘𜷙𜷚▄𜷛𜷜𜷝𜷞▙𜷟𜷠𜷡𜷢▟𜷣▆𜷤𜷥█"
    #partial infill pixels(pips) glyxel look up table, columns x rows
    pips_glut =  [[],["","",""],["","","","",""]]
    pips_glut[1][1] = " ●"
    pips_glut[1][2] = " ᛫.:"
    pips_glut[2][2] = " 𜰡𜰢𜰣𜰤𜰥𜰦𜰧𜰨𜰩𜰪𜰫𜰬𜰭𜰮𜰯"
    pips_glut[2][3] = " 𜹑𜹒𜹓𜹔𜹕𜹖𜹗𜹘𜹙𜹚𜹛𜹜𜹝𜹞𜹟𜹠𜹡𜹢𜹣𜹤𜹥𜹦𜹧𜹨𜹩𜹪𜹫𜹬𜹭𜹮𜹯𜹰𜹱𜹲𜹳𜹴𜹵𜹶𜹷𜹸𜹹𜹺𜹻𜹼𜹽𜹾𜹿𜺀𜺁𜺂𜺃𜺄𜺅𜺆𜺇𜺈𜺉𜺊𜺋𜺌𜺍𜺎𜺏"
    pips_glut[2][4] = "⠀⠁⠈⠉⠂⠃⠊⠋⠐⠑⠘⠙⠒⠓⠚⠛⠄⠅⠌⠍⠆⠇⠎⠏⠔⠕⠜⠝⠖⠗⠞⠟⠠⠡⠨⠩⠢⠣⠪⠫⠰⠱⠸⠹⠲⠳⠺⠻⠤⠥⠬⠭⠦⠧⠮⠯⠴⠵⠼⠽⠶⠷⠾⠿⡀⡁⡈⡉⡂⡃⡊⡋⡐⡑⡘⡙⡒⡓⡚⡛⡄⡅⡌⡍⡆⡇⡎⡏⡔⡕⡜⡝⡖⡗⡞⡟⡠⡡⡨⡩⡢⡣⡪⡫⡰⡱⡸⡹⡲⡳⡺⡻⡤⡥⡬⡭⡦⡧⡮⡯⡴⡵⡼⡽⡶⡷⡾⡿⢀⢁⢈⢉⢂⢃⢊⢋⢐⢑⢘⢙⢒⢓⢚⢛⢄⢅⢌⢍⢆⢇⢎⢏⢔⢕⢜⢝⢖⢗⢞⢟⢠⢡⢨⢩⢢⢣⢪⢫⢰⢱⢸⢹⢲⢳⢺⢻⢤⢥⢬⢭⢦⢧⢮⢯⢴⢵⢼⢽⢶⢷⢾⢿⣀⣁⣈⣉⣂⣃⣊⣋⣐⣑⣘⣙⣒⣓⣚⣛⣄⣅⣌⣍⣆⣇⣎⣏⣔⣕⣜⣝⣖⣗⣞⣟⣠⣡⣨⣩⣢⣣⣪⣫⣰⣱⣸⣹⣲⣳⣺⣻⣤⣥⣬⣭⣦⣧⣮⣯⣴⣵⣼⣽⣶⣷⣾⣿"


    @staticmethod
    def pane2strips(
            style: Style|None,
            basis,
            pane,
            pips ) -> List[List[Segment]]:
        x,y,mask = pane
        if x == 0 or y == 0: return [ Strip.blank(0) ]

        glut = ToGlyxels.pips_glut if pips else ToGlyxels.full_glut

        selection = Image.new( '1', (x,y) )
        selection.putdata(mask)
        #glyph based pixels must be an integer multiple of glyph cell basis, ie. 2x4 -> octants
        while x % basis[0] != 0: x += 1
        while y % basis[1] != 0: y += 1
        slate = Image.new( '1', (x,y) )
        #place bitmap into upper left corner
        slate.paste( selection, (0,0) )

        base_row = int( y/basis[1] - 1 )
        mid_row = int( base_row/2 )
        cap_row = 0
        
        strips = []
        for y_glyph in range( 0, y, basis[1] ):
            y_strip = []
            y_row = int( y_glyph/basis[1] )
            y_style = ToGlyxels._y_Style( style, cap_row, mid_row, base_row, y_row)
            for x_glyph in range( 0, x, basis[0] ):
                glyph_idx = 0
                glyxelList = []
                for y_idx in range( basis[1] ):
                    for x_idx in range( basis[0] ):
                        glyxelList.append( slate.getpixel( (x_glyph+x_idx, y_glyph+y_idx) ) )
                for exp, gColor in enumerate( glyxelList ):
                    if gColor > 0:
                        glyph_idx += 2**exp
                glyph = glut[basis[0]][basis[1]][glyph_idx]
                y_strip.append( Segment( glyph, y_style ) )
            strips.append( Strip(y_strip) )
        return strips

    @staticmethod
    def style_strips( strips, style):
        base_row = len( strips )
        mid_row = int( base_row/2 )
        cap_row = 0
        new_strips = []
        for y_row, y_strip in enumerate( strips ):
            y_style = ToGlyxels._y_Style( style, cap_row, mid_row, base_row, y_row)
            new_strips.append( y_strip.apply_style( y_style ) )
        return new_strips

            
    @staticmethod
    def _y_Style( style, cap_row, mid_row, base_row, y_row):
        if style:
            if style.overline and y_row != cap_row:
                style = style + Style(overline=False)
            if style.strike and y_row != mid_row:
                style = style + Style(strike=False)
            if y_row != base_row:
                if style.underline:
                    style = style + Style(underline=False)
                if style.underline2:
                    style = style + Style(underline2=False)
        return style


    @staticmethod
    def _chunk_join( strips, chunk ):
        if len( strips ) == 0: return chunk
        joint = []
        for idx, line in enumerate( strips ):
            joint.append(Strip.join( (line,chunk[idx]) ).simplify())
        return joint

    @staticmethod
    def from_renderable( 
            phrase,
            basis = (2,4), 
            pips = False,
            font_size: int = 12,
            font_name: str = "TerminusTTF-4.46.0.ttf"
            ) -> List[List[Segment]]:
        strips = []
        con_strips = Console().render_lines( phrase, pad=False )
        for a_strip in con_strips:
            for seg in a_strip:
                pane = ToGlyxels._font_pane( seg.text, font_name, font_size )
                chunk = ToGlyxels.pane2strips( seg.style, basis, pane, pips )
                strips = ToGlyxels._chunk_join( strips, chunk )
        return strips

    @staticmethod
    def from_string( 
            phrase: str,
            basis = (2,4), 
            pips = False,
            font_size: int = 12,
            font_name: str = "TerminusTTF-4.46.0.ttf"
            ) -> List[List[Segment]]:
        pane = ToGlyxels._font_pane( phrase, font_name, font_size )
        return ToGlyxels.pane2strips( None, basis, pane, pips )

    @staticmethod
    def _font_pane( phrase, font_name, font_size ):
        font_asset = resources.files().joinpath( "assets", font_name )
        font = ImageFont.truetype( font_asset, size=font_size )
        l,t,r,b = font.getbbox( phrase )
        mask = [x for x in font.getmask(phrase, mode='1')]
        return (r,b,mask)




from .toglyxels import ToGlyxels

from textual.reactive import reactive
from textual.strip import Strip
from textual.widget import Widget

from rich.console import RenderableType
from rich.segment import Segment
from rich.style import Style
from rich.text import Text
from rich.traceback import install
install()

class EnGlyph( Widget, inherit_bindings=False ):
    DEFAULT_CSS = """
    EnGlyph {
        height: auto;
        width: auto;
    }
    """

    def __init__( self,
                 renderable: RenderableType = "",
                 *,
                 basis = (2,4),
                 pips = False,
                 markup: bool = True,
                 name: str | None = None,
                 id: str | None = None,
                 classes: str | None = None,
                 disabled: bool = False,
                 ) -> None:
        super().__init__(name=name, id=id, classes=classes, disabled=disabled)
        self.renderable = self._enRich_string( renderable, markup )
        self.basis = basis
        self.pips = pips
        self._strips_cache = ToGlyxels.from_renderable( self.renderable, self.basis, self.pips )

    def get_content_width(self, a, b):
        return self._strips_cache[0].cell_length

    def get_content_height(self, a, b, c):
        return len( self._strips_cache )

    def _enRich_string(self, renderable: RenderableType, markup: bool=True) -> RenderableType:
        if isinstance(renderable, str):
            if markup:
                renderable = Text.from_markup(renderable)
            else:
                renderable = Text(renderable)
        return renderable

    def update( self, renderable: str|None = None, basis: str|None = None, pips: bool|None = None ) -> None:
        """New display input"""
        self.renderable = renderable or self.renderable
        self.basis = basis or self.basis
        self.pips = pips or self.pips
        self.refresh(layout=True)

    def render_line( self, row:int ) -> Strip:
        strip = Strip.blank(0)
        if row == 0:
            self.renderable.stylize_before( self.rich_style )
            self._strips_cache = ToGlyxels.from_renderable( self.renderable, self.basis, self.pips )
        if self._strips_cache is not None and row < len( self._strips_cache ):
            strip = self._strips_cache[row]

        return strip

